//
//  Toolkit.h
//  Toolkit
//
//  Copyright (c) 2012 Interface co.,ltd. All rights reserved.
//

#import <Foundation/Foundation.h>

#if 0	// CFPluginCOM.hの定義と競合するため変更
typedef unsigned long ULONG;
#else
typedef UInt32 ULONG;
#endif
typedef unsigned int  UINT;

typedef int TkError;
enum {
    TKErrSuccess = 0,
    TKErrInvalidParameter = -2,
    TKErrNotInitialized = -1,
    TKErrInternal = -3,
    TKErrNoDevice = -4,
    TKErrCancelled = -5,
    TKErrNotSupported = -99,
};

enum TKNotifyError
{
    TKNotifyErrorDeviceDisconnected = 0,
    TKNotifyErrorNoDevice           = 1,
    TKNotifyErrorNonPlayableFile    = 2,
    TKNotifyErrorDecodeFailed       = 3,
    TKNotifyErrorInsufficientMemory = 4,
};

enum TKRepeatMode
{
    TKRepeatModeNone = 0,
    TKRepeatModeTitle = 1,
    TKRepeatModePlaylist = 2,
};

enum PlaybackState
{
    PbStateStop = 0,
    PbStatePlayback = 1,
    PbStatePause = 2,
};

enum TkPlaylistStateChange
{
    TkPlaylistStateChangeTotalDuration = 0,
    TkPlaylistStateChangeReadonly = 1,
    TkPlaylistStateChangeModifiable = 2,
};

@interface AudioFileInfo : NSObject
{
    NSString* fileName;
    NSString* titleName;
    NSString* artistName;
    NSString* extName;
    ULONG samplingRate;
    ULONG bitResolution;
    double durationBySec;
}
@property (retain) NSString* fileName;
@property (retain) NSString* titleName;
@property (retain) NSString* artistName;
@property (retain) NSString* extName;
@property (assign) ULONG samplingRate;
@property (assign) ULONG bitResolution;
@property (assign) double durationBySec;
@end

@protocol ITFAudioToolkitDelegate

- (void)songSwitch;
- (void)stateDidChange:(enum PlaybackState)state;
- (void)notifyError:(enum TKNotifyError)error info:(AudioFileInfo*)info;
- (void)notifyPos:(ULONG)pos_title total:(ULONG)pos_total;
- (void)playlistStateDidChange:(enum TkPlaylistStateChange)reason;
- (void)playlistSelectionDidChange:(bool)select;
- (void)propertiesMenuSelected;
- (void)checkForLastConnectedDevice;

@end

#define LAST_SELECTED_DEVICE_NANE_KEY  (@"Last Selected Output Device Name")

#define DECODE_MODE_SELECTION_INDEX_KEY (@"Decode Mode Selection Index")
#define SELECTED_DECODE_MODE_DOP        (0)
#define SELECTED_DECODE_MODE_DSD_NATIVE (1)

#define RAM_MODE_SELECTION_INDEX_KEY   (@"RAM Mode Selection Index")
#define SELECTED_PLAY_MODE_NORMAL (0)
#define SELECTED_PLAY_MODE_RAM (1)

#define RAM_MODE_MEMORY_LIMITAITON_KEY (@"RAM Mode Memory Limitation")
#define RAM_MODE_MEMORY_LIMITATION_DEFAULT (100) // RAMモードのメモリ上限
#define RAM_MODE_MEMORY_LIMITATION_COEFF (1024 * 1024)

#define GAPLESS_THREASHOLD_DB_KEY (@"Gapless Threashold DB")

#define GAPLESS_ENABLE_KEY        (@"MP3 gapless play enable")
#define GAPLESS_THREASHOLD_DB_DEFAULT (-45.)
#define GAPLESS_THREASHOLD_DB_MAX (0.0)
#define GAPLESS_THREASHOLD_DB_MIN (-192.0)
#define GAPLESS_THRESHOLD_NONE    (0.) /**< ギャップレス処理OFF */

#define LAST_PLAYLIST_KEY (@"Last Playlist")

typedef NSMutableDictionary TkConfig;
@interface NSMutableDictionary (TkConfig)
@property (retain) NSString* deviceName;
@property (assign) int decodeMode;
@property (assign) int ramMode;
@property (assign) int ramModeMemoryLimit;
@property (assign) bool gaplessMode;
@property (assign) int gaplessCutoff;
@end


#define DEVICE_INFO_NAME_KEY        (@"name")
#define DEVICE_INFO_DOP_KEY         (@"DoP")
#define DEVICE_INFO_DSD_NATIVE_KEY  (@"DSD Native")

typedef NSDictionary TkDeviceInfo;
@interface NSDictionary (TkDeviceInfo)
@property (readonly) NSString* deviceName;
@property (readonly) bool isDoPSupported;
@property (readonly) bool isDSDNativeSupported;
@end


@interface TkPluginInfoItem : NSObject
{
    NSString* name;
    NSString* exts;
    NSString* vers;
}
- (id)initWithName:(NSString*)aName exts:(NSString*)aExt version:(NSString*)aVersion;
@property (readonly) NSString* pluginName;
@property (readonly) NSString* pluginExt;
@property (readonly) NSString* pluginVersion;
@end


typedef struct _PlaylistSettingInfo
{
	/**
	 * @brief プレイリストの親となるビュー(NSView)のインスタンス
	 */
    void* parent;
    int left;
    int top;
    int width;
    int height;
    /**
     * @brief 各カラム要素の表示幅
     * [0]: Playing
     * [1]: Artist
     * [2]: Title
     * [3]: Duration
     */
    int column_width[4];
} PlaylistSettingInfo;

TkError Toolkit_Init(id <ITFAudioToolkitDelegate> delegate);
void Toolkit_Exit(void);
TkError Toolkit_OpenFile(void);
TkError Toolkit_OpenCD(void);
TkError Toolkit_AddFile(void);
TkError Toolkit_AddDroppedFiles(NSArray *paths);
TkError Toolkit_RemoveFile(void);
TkError Toolkit_NewPlaylist(void);
TkError Toolkit_LoadPlaylist(void);
TkError Toolkit_SavePlaylist(void);
TkError Toolkit_PlayRequest(void);
TkError Toolkit_StopRequest(void);
TkError Toolkit_PauseRequest(void);
TkError Toolkit_NextRequest(void);
TkError Toolkit_PrevRequest(void);
TkError Toolkit_Shuffle(bool onoff);
TkError Toolkit_Repeat(int mode);
TkError Toolkit_SetPos(UINT pos);
TkError Toolkit_GetPlaylist(PlaylistSettingInfo* info);
TkError Toolkit_SetPlaylist(PlaylistSettingInfo* info);
TkError Toolkit_GetFileInfo(bool select, AudioFileInfo * __autoreleasing *info);
TkError Toolkit_TotalTime(ULONG* time);
TkError Toolkit_GetDeviceList(NSArray* __autoreleasing *name);
TkError Toolkit_GetNumberOfSupportDevices(UINT* count);
TkError Toolkit_GetConfiguration(TkConfig *config);
TkError Toolkit_SetConfiguration(TkConfig *config);
TkError Toolkit_GetDebugMode(bool *debug);
TkError Toolkit_GetToolkitVersion(NSString* __autoreleasing * version);
TkError Toolkit_GetPluginVersion(NSArray* __autoreleasing * info);
TkError Toolkit_saveLastPlaylist();
TkError Toolkit_loadLastPlaylist();

BOOL Toolkit_IsPlaylistDirty(void);
TkError Toolkit_GetPlaylistItemCount(int* count);

id getObject(void);
